@@ -21,6 +21,8 @@ enum {
     PROP_0,
     PROP_DEFAULT_SESSION_HINT,
     PROP_HIDE_USERS_HINT,
+    PROP_SHOW_MANUAL_LOGIN_HINT,
+    PROP_LOCK_HINT,
     PROP_HAS_GUEST_ACCOUNT_HINT,
     PROP_SELECT_USER_HINT,
     PROP_SELECT_GUEST_HINT,
@@ -118,22 +120,6 @@ int_length ()
     return 4;
 }
 
-static void
-write_message (LightDMGreeter *greeter, guint8 *message, gsize message_length)
-{
-    LightDMGreeterPrivate *priv = GET_PRIVATE (greeter);
-    GIOStatus status;
-    GError *error = NULL;
-
-    status = g_io_channel_write_chars (priv->to_server_channel, (gchar *) message, message_length, NULL, &error);
-    if (error)
-        g_warning ("Error writing to daemon: %s", error->message);
-    g_clear_error (&error);
-    if (status == G_IO_STATUS_NORMAL)
-        g_debug ("Wrote %zi bytes to daemon", message_length);
-    g_io_channel_flush (priv->to_server_channel, NULL);
-}
-
 static void
 write_int (guint8 *buffer, gint buffer_length, guint32 value, gsize *offset)
 {
@@ -223,6 +209,35 @@ get_message_length (guint8 *message, gsize message_length)
     return read_int (message, message_length, &offset);
 }
 
+static void
+write_message (LightDMGreeter *greeter, guint8 *message, gsize message_length)
+{
+    LightDMGreeterPrivate *priv = GET_PRIVATE (greeter);
+    GIOStatus status;
+    GError *error = NULL;
+    guint32 stated_length;
+
+    /* Double check that we're sending well-formed messages.  If we say we're
+       sending more than we do, we end up DOS'ing lightdm as it waits for the
+       rest.  If we say we're sending less than we do, we confuse the heck out
+       of lightdm, as it starts reading headers from the middle of our
+       messages. */
+    stated_length = HEADER_SIZE + get_message_length (message, message_length);
+    if (stated_length != message_length)
+    {
+        g_warning ("Refusing to write malformed packet to daemon: declared size is %u, but actual size is %zu", stated_length, message_length);
+        return;
+    }
+
+    status = g_io_channel_write_chars (priv->to_server_channel, (gchar *) message, message_length, NULL, &error);
+    if (error)
+        g_warning ("Error writing to daemon: %s", error->message);
+    g_clear_error (&error);
+    if (status == G_IO_STATUS_NORMAL)
+        g_debug ("Wrote %zi bytes to daemon", message_length);
+    g_io_channel_flush (priv->to_server_channel, NULL);
+}
+
 static void
 handle_connected (LightDMGreeter *greeter, guint8 *message, gsize message_length, gsize *offset)
 {
@@ -545,7 +560,13 @@ lightdm_greeter_get_default_session_hint (LightDMGreeter *greeter)
  * lightdm_greeter_get_hide_users_hint:
  * @greeter: A #LightDMGreeter
  *
- * Check if user accounts should be shown.
+ * Check if user accounts should be shown.  If this is TRUE then the list of
+ * accounts should be taken from #LightDMUserList and displayed in the greeter
+ * for the user to choose from.  Note that this list can be empty and it is
+ * recommended you show a method for the user to enter a username manually.
+ * 
+ * If this option is shown the greeter should only allow these users to be
+ * chosen for login unless the manual login hint is set.
  *
  * Return value: #TRUE if the available users should not be shown.
  */
@@ -560,6 +581,47 @@ lightdm_greeter_get_hide_users_hint (LightDMGreeter *greeter)
     return g_strcmp0 (value, "true") == 0;
 }
 
+/**
+ * lightdm_greeter_get_show_manual_login_hint:
+ * @greeter: A #LightDMGreeter
+ *
+ * Check if a manual login option should be shown.  If set the GUI
+ * should provide a way for a username to be entered manually.
+ * Without this hint a greeter which is showing a user list can
+ * limit logins to only those users.
+ *
+ * Return value: #TRUE if a manual login option should be shown.
+ */
+gboolean
+lightdm_greeter_get_show_manual_login_hint (LightDMGreeter *greeter)
+{
+    const gchar *value;
+
+    g_return_val_if_fail (LIGHTDM_IS_GREETER (greeter), FALSE);
+    value = lightdm_greeter_get_hint (greeter, "show-manual-login");
+
+    return g_strcmp0 (value, "true") == 0;
+}
+
+/**
+ * lightdm_greeter_get_lock_hint:
+ * @greeter: A #LightDMGreeter
+ *
+ * Check if the greeter is acting as a lock screen.
+ *
+ * Return value: #TRUE if the greeter was triggered by locking the seat.
+ */
+gboolean
+lightdm_greeter_get_lock_hint (LightDMGreeter *greeter)
+{
+    const gchar *value;
+
+    g_return_val_if_fail (LIGHTDM_IS_GREETER (greeter), FALSE);
+    value = lightdm_greeter_get_hint (greeter, "lock-screen");
+
+    return g_strcmp0 (value, "true") == 0;
+}
+
 /**
  * lightdm_greeter_get_has_guest_account_hint:
  * @greeter: A #LightDMGreeter
@@ -970,6 +1032,12 @@ lightdm_greeter_get_property (GObject    *object,
     case PROP_HIDE_USERS_HINT:
         g_value_set_boolean (value, lightdm_greeter_get_hide_users_hint (self));
         break;
+    case PROP_SHOW_MANUAL_LOGIN_HINT:
+        g_value_set_boolean (value, lightdm_greeter_get_show_manual_login_hint (self));
+        break;
+    case PROP_LOCK_HINT:
+        g_value_set_boolean (value, lightdm_greeter_get_lock_hint (self));
+        break;
     case PROP_HAS_GUEST_ACCOUNT_HINT:
         g_value_set_boolean (value, lightdm_greeter_get_has_guest_account_hint (self));
         break;
@@ -1078,7 +1146,23 @@ lightdm_greeter_class_init (LightDMGreeterClass *klass)
                                      PROP_HIDE_USERS_HINT,
                                      g_param_spec_boolean ("hide-users-hint",
                                                            "hide-users-hint",
-                                                           "hide users hint",
+                                                           "Hide users hint",
+                                                           FALSE,
+                                                           G_PARAM_READABLE));
+
+    g_object_class_install_property (object_class,
+                                     PROP_SHOW_MANUAL_LOGIN_HINT,
+                                     g_param_spec_boolean ("show-manual-login-hint",
+                                                           "show-manual-login-hint",
+                                                           "Show manual login hint",
+                                                           FALSE,
+                                                           G_PARAM_READABLE));
+
+    g_object_class_install_property (object_class,
+                                     PROP_LOCK_HINT,
+                                     g_param_spec_boolean ("lock-hint",
+                                                           "lock-hint",
+                                                           "Lock hint",
                                                            FALSE,
                                                            G_PARAM_READABLE));
 
