@@ -93,14 +93,20 @@ const gchar *lightdm_user_get_home_directory (LightDMUser *user);
 
 const gchar *lightdm_user_get_image (LightDMUser *user);
 
+const gchar *lightdm_user_get_background (LightDMUser *user);
+
 const gchar *lightdm_user_get_language (LightDMUser *user);
 
 const gchar *lightdm_user_get_layout (LightDMUser *user);
 
+const gchar * const *lightdm_user_get_layouts (LightDMUser *user);
+
 const gchar *lightdm_user_get_session (LightDMUser *user);
 
 gboolean lightdm_user_get_logged_in (LightDMUser *user);
 
+gboolean lightdm_user_get_has_messages (LightDMUser *user);
+
 G_END_DECLS
 
 #endif /* _LIGHTDM_USER_H_ */
