@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Robert Ancell.
+ * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
  * 
  * This program is free software: you can redistribute it and/or modify it under
@@ -13,7 +13,8 @@
 #define _DISPLAY_MANAGER_H_
 
 #include <glib-object.h>
-#include "display.h"
+
+#include "seat.h"
 
 G_BEGIN_DECLS
 
@@ -32,19 +33,23 @@ typedef struct
 {
     GObjectClass parent_class;
 
-    void (*display_added)(DisplayManager *manager, Display *display);
+    void (*seat_added)(DisplayManager *manager, Seat *seat);
+    void (*seat_removed)(DisplayManager *manager, Seat *seat);
+    void (*stopped)(DisplayManager *manager);
 } DisplayManagerClass;
 
 GType display_manager_get_type (void);
 
-DisplayManager *display_manager_new (GKeyFile *config);
+DisplayManager *display_manager_new (void);
 
-Display *display_manager_add_display (DisplayManager *manager);
+gboolean display_manager_add_seat (DisplayManager *manager, Seat *seat);
 
-void display_manager_switch_to_user (DisplayManager *manager, char *username);
+GList *display_manager_get_seats (DisplayManager *manager);
 
 void display_manager_start (DisplayManager *manager);
 
+void display_manager_stop (DisplayManager *manager);
+
 G_END_DECLS
 
 #endif /* _DISPLAY_MANAGER_H_ */
