@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Robert Ancell.
+ * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
  * 
  * This program is free software: you can redistribute it and/or modify it under
@@ -13,6 +13,7 @@
 #define _XDMCP_SESSION_PRIVATE_H_
 
 #include "xdmcp-server.h"
+#include "xauthority.h"
 
 struct XDMCPSessionPrivate
 {
@@ -23,14 +24,10 @@ struct XDMCPSessionPrivate
     gchar *manufacturer_display_id;
 
     GInetAddress *address;
-    GInetAddress *address6;  
 
     guint inactive_timeout;
 
-    gchar *authorization_name;
-  
-    guchar *authorization_data;
-    gsize authorization_data_length;
+    XAuthority *authority;
   
     gboolean started;
 
